@@ -41,6 +41,7 @@ __license__ = "MIT License <http://www.opensource.org/licenses/mit>"
 __contact__ = "devel@lists.lv2plug.in"
 
 import datetime
+import optparse
 import os
 import re
 import sys
@@ -227,6 +228,9 @@ if have_pygments:
         }
 
 def linkify(string):
+    if linkmap == {}:
+        return string
+
     "Add links to code documentation for identifiers"
     if string in linkmap.keys():
         # Exact match for complete string
@@ -287,7 +291,9 @@ def getComment(m, urinode, classlist, proplist, instalist):
             curie = match.group(0)
             uri   = rdflib.URIRef(spec_ns + name)
             if prefix == spec_pre:
-                if not (uri in classlist or uri in instalist or uri in proplist):
+                if not ((classlist and uri in classlist) or
+                        (instalist and uri in instalist) or
+                        (proplist and uri in proplist)):
                     print("warning: Link to undefined resource <%s>\n" % text)
                 return '<a href="#%s">%s</a>' % (name, curie)
             elif prefix in namespaces:
@@ -721,6 +727,13 @@ def buildIndex(m, classlist, proplist, instalist=None):
 
     azlist = '<dl class="index">'
 
+    def termLink(m, t):
+        if str(t).startswith(spec_ns_str):
+            name = termName(m, t)
+            return '<a href="#%s">%s</a>' % (name, name)
+        else:
+            return '<a href="%s">%s</a>' % (str(t), str(t))
+        
     if (len(classlist) > 0):
         azlist += "<dt>Classes</dt><dd><ul>"
         classlist.sort()
@@ -739,10 +752,7 @@ def buildIndex(m, classlist, proplist, instalist=None):
                 continue
 
             shown[c] = True
-            name = termName(m, c)
-            if name.startswith(spec_ns_str):
-                name = name.split(spec_ns_str[-1])[1]
-            azlist += '<li><a href="#%s">%s</a>' % (name, name)
+            azlist += '<li>' + termLink(m, c)
             def class_tree(c):
                 tree = ''
                 shown[c] = True
@@ -753,8 +763,7 @@ def buildIndex(m, classlist, proplist, instalist=None):
                 subclasses.sort()
 
                 for s in subclasses:
-                    s_name = termName(m, s)
-                    tree += '<li><a href="#%s">%s</a>\n' % (s_name, s_name)
+                    tree += '<li>' + termLink(m, s)
                     tree += class_tree(s)
                     tree += '</li>'
                 if tree != '':
@@ -769,10 +778,7 @@ def buildIndex(m, classlist, proplist, instalist=None):
         proplist.sort()
         props = []
         for p in proplist:
-            name = termName(m, p)
-            if name.startswith(spec_ns_str):
-                name = name.split(spec_ns_str[-1])[1]
-            props += ['<a href="#%s">%s</a>' % (name, name)]
+            props += [termLink(m, p)]
         azlist += ', '.join(props) + '</dd>\n'
 
     if (instalist != None and len(instalist) > 0):
@@ -998,6 +1004,9 @@ def getInstances(model, classes, properties):
 def load_tags(path, docdir):
     "Build a (symbol => URI) map from a Doxygen tag file."
 
+    if not path or not docdir:
+        return {}
+
     def getChildText(elt, tagname):
         "Return the content of the first child node with a certain tag name."
         for e in elt.childNodes:
@@ -1037,7 +1046,7 @@ def load_tags(path, docdir):
 
     return linkmap
 
-def specgen(specloc, indir, style_uri, docdir, tags, instances=False, offline=False):
+def specgen(specloc, indir, style_uri, docdir, tags, opts, instances=False, offline=False):
     """The meat and potatoes: Everything starts here."""
 
     global spec_url
@@ -1061,7 +1070,8 @@ def specgen(specloc, indir, style_uri, docdir, tags, instances=False, offline=Fa
 
     m = rdflib.ConjunctiveGraph()
     manifest_path = os.path.join(os.path.dirname(specloc), 'manifest.ttl')
-    m.parse(manifest_path, format='n3')
+    if os.path.exists(manifest_path):
+        m.parse(manifest_path, format='n3')
     m.parse(specloc, format='n3')
 
     bundle_path = os.path.split(specloc[specloc.find(':') + 1:])[0]
@@ -1142,9 +1152,17 @@ def specgen(specloc, indir, style_uri, docdir, tags, instances=False, offline=Fa
     template = template.replace('@REFERENCE@', termlist)
     template = template.replace('@FILENAME@', filename)
     template = template.replace('@HEADER@', basename + '.h')
-    template = template.replace('@MAIL@', 'devel@lists.lv2plug.in')
     template = template.replace('@HISTORY@', specHistory(m, spec))
 
+    mail_row = ''
+    if 'list_email' in opts:
+        mail_row = '<tr><th>Discuss</th><td><a href="mailto:%s">%s</a>' % (
+            opts['list_email'], opts['list_email'])
+        if 'list_page' in opts:
+            mail_row += ' <a href="%s">(subscribe)</a>' % opts['list_page']
+        mail_row += '</td></tr>'
+    template = template.replace('@MAIL@', mail_row)
+
     version = specVersion(m, spec)  # (minor, micro, date)
     date_string = version[2]
     if date_string == "":
@@ -1241,7 +1259,7 @@ def getOntologyNS(m):
 
 def usage():
     script = os.path.basename(sys.argv[0])
-    print("""Usage: %s ONTOLOGY INDIR STYLE OUTPUT DOCDIR TAGS [FLAGS]
+    return """Usage: %s ONTOLOGY INDIR STYLE OUTPUT [DOCDIR TAGS] [FLAGS]
 
         ONTOLOGY  : Path to ontology file
         INDIR     : Input directory containing template.html and style.css
@@ -1256,42 +1274,34 @@ def usage():
 
 Example:
     %s lv2_foos.ttl template.html style.css lv2_foos.html ../doc -i -p foos
-""" % (script, script))
-    sys.exit(-1)
-
+""" % (script, script)
 
 if __name__ == "__main__":
     """Ontology specification generator tool"""
 
-    args = sys.argv[1:]
+    opt = optparse.OptionParser(usage=usage(),
+                                description='Write HTML documentation for an RDF ontology.')
+    opt.add_option('--list-email', type='string', dest='list_email')
+    opt.add_option('--list-page', type='string', dest='list_page')
+    opt.add_option('-i', action='store_true', dest='instances')
+    opt.add_option('-p', type='string', dest='prefix')
+
+    (options, args) = opt.parse_args()
+    opts = vars(options)
+
     if (len(args) < 3):
-        usage()
-    else:
-        ontology = "file:" + str(args[0])
-        indir    = args[1]
-        style    = args[2]
-        output   = args[3]
-        docdir   = args[4]
-        tags     = args[5]
+        print(usage())
+        sys.exit(-1)
+        
+    spec_pre = options.prefix
+    ontology = "file:" + str(args[0])
+    indir    = args[1]
+    style    = args[2]
+    output   = args[3]
+    docdir   = None
+    tags     = None
+    if len(args) > 5:
+        docdir = args[4]
+        tags   = args[5]
 
-        # Flags
-        instances = False
-        if len(args) > 7:
-            flags = args[7:]
-            i = 0
-            while i < len(flags):
-                if flags[i] == '-i':
-                    instances = True
-                elif flags[i] == '-p':
-                    spec_pre = flags[i + 1]
-                    i += 1
-                i += 1
-
-    try:
-        save(output, specgen(ontology, indir, style, docdir, tags, instances=instances))
-    except:
-        e = sys.exc_info()[1]
-        print('error: ' + str(e))
-        sys.exit(1)
-
-    sys.exit(0)
+    save(output, specgen(ontology, indir, style, docdir, tags, opts, instances=options.instances))
